@@ -198,9 +198,10 @@ struct StringXDefine  Operation_Type[] = {
   {"Update"                 , OPERATION_UPDATE},
   {"Update_Translation"     , OPERATION_UPDATETRANSLATION},
   {"Solve"                  , OPERATION_SOLVE},
+  {"SolveAgain"             , OPERATION_SOLVEAGAIN},
   {"GenerateJac"            , OPERATION_GENERATEJAC},
   {"SolveJac"               , OPERATION_SOLVEJAC},
-  {"SolveAgain"             , OPERATION_SOLVEAGAIN},
+  {"SolveJacAgain"          , OPERATION_SOLVEJACAGAIN},
   {"GenerateSeparate"       , OPERATION_GENERATESEPARATE},
   {"InitSolution"           , OPERATION_INITSOLUTION},
   {"SaveSolution"           , OPERATION_SAVESOLUTION},
@@ -211,6 +212,7 @@ struct StringXDefine  Operation_Type[] = {
   {"SolveJac_AdaptRelax"    , OPERATION_SOLVEJACADAPTRELAX}, 
   {"SaveSolutionExtendedMH" , OPERATION_SAVESOLUTIONEXTENDEDMH},
   {"DummyDofs"              , OPERATION_DUMMYDOFS},
+  {"InitCorrection"         , OPERATION_INITCORRECTION},
   {NULL                     , OPERATION_NONE}
 } ;
 
