@@ -26,7 +26,7 @@
 extern struct Problem Problem_S ;
 extern struct CurrentData Current ;
 
-FILE  * File_PRE, * File_RES, * File_TMP ;
+FILE  * File_PRE = 0, * File_RES = 0, * File_TMP = 0 ;
 
 struct DofData  * CurrentDofData ;
 
@@ -107,6 +107,16 @@ void Dof_SetCurrentDofData(struct DofData * DofData_P)
 
 void Dof_OpenFile(int Type, char * Name, const char * Mode) 
 {
+  if(Msg::GetCommRank() && (Mode[0] == 'w' || Mode[0] == 'a')){
+    switch (Type) {
+    case DOF_PRE :  File_PRE = 0 ;  break ;
+    case DOF_RES :  File_RES = 0 ;  break ;
+    case DOF_TMP :  File_RES = 0 ;  break ;
+    default      :  break ;
+    }
+    return;
+  }
+  
   const char  * Extension;
   char    FileName[256] ;
   FILE  * File_X ;
@@ -138,9 +148,9 @@ void Dof_OpenFile(int Type, char * Name, const char * Mode)
 void Dof_CloseFile(int Type)
 {
   switch (Type) {
-  case DOF_PRE :  fclose(File_PRE) ;  break ;
-  case DOF_RES :  fclose(File_RES) ;  break ;
-  case DOF_TMP :  fclose(File_RES) ;  File_RES = File_TMP ; break ;
+  case DOF_PRE :  if(File_PRE) fclose(File_PRE) ;  break ;
+  case DOF_RES :  if(File_RES) fclose(File_RES) ;  break ;
+  case DOF_TMP :  if(File_RES) fclose(File_RES) ;  File_RES = File_TMP ; break ;
   }
 }
 
@@ -163,6 +173,8 @@ void Dof_FlushFile(int Type)
 void Dof_WriteFilePRE0(int Num_Resolution, char * Name_Resolution,
 		       int Nbr_DofData)
 {
+  if(Msg::GetCommRank()) return;
+
   fprintf(File_PRE, "$Resolution /* '%s' */\n", Name_Resolution) ;
   fprintf(File_PRE, "%d %d\n", Num_Resolution, Nbr_DofData) ;
   fprintf(File_PRE, "$EndResolution\n") ;
@@ -174,6 +186,8 @@ void Dof_WriteFilePRE0(int Num_Resolution, char * Name_Resolution,
 
 void Dof_ReadFilePRE0(int * Num_Resolution, int * Nbr_DofData)
 {
+  Msg::Barrier();
+
   char  String[256] ;
 
   do { 
@@ -199,6 +213,8 @@ void Dof_ReadFilePRE0(int * Num_Resolution, int * Nbr_DofData)
 
 void Dof_WriteFilePRE(struct DofData * DofData_P)
 {
+  if(Msg::GetCommRank()) return;
+
   int  i, Nbr_Index ;
   struct Dof  * Dof_P0 ;
 
@@ -296,6 +312,8 @@ void Dof_WriteDofPRE(void * a, void * b)
 
 void Dof_ReadFilePRE(struct DofData * DofData_P)
 {
+  Msg::Barrier();
+
   int         i, Nbr_Index, Int, Dummy ;
   struct Dof  Dof ;
   char        String[256] ;
@@ -393,6 +411,8 @@ void Dof_ReadFilePRE(struct DofData * DofData_P)
 
 void Dof_WriteFileRES0(char * Name_File, int Format)
 {
+  if(Msg::GetCommRank()) return;
+
   Dof_OpenFile(DOF_RES, Name_File, (char*)(Format ? "wb" : "w")) ;
   fprintf(File_RES, "$ResFormat /* GetDP %s, %s */\n", GETDP_VERSION, 
 	  Format ? "binary" : "ascii") ;
@@ -408,6 +428,8 @@ void Dof_WriteFileRES0(char * Name_File, int Format)
 void Dof_WriteFileRES_ExtendMH(char * Name_File, struct DofData * DofData_P,
 			       int Format, int NbrH)
 {
+  if(Msg::GetCommRank()) return;
+
   gVector x;
   double d;
   int i, inew;
@@ -441,6 +463,8 @@ void Dof_WriteFileRES_ExtendMH(char * Name_File, struct DofData * DofData_P,
 void Dof_WriteFileRES_MHtoTime(char * Name_File, struct DofData * DofData_P, 
 			       int Format, List_T * Time_L) 
 {
+  if(Msg::GetCommRank()) return;
+
   gVector x;
   double Time, d1, d2, d, *Pulsation;
   int iT, i, j, k;
@@ -492,11 +516,13 @@ void Dof_WriteFileRES_MHtoTime(char * Name_File, struct DofData * DofData_P,
 void Dof_WriteFileRES(char * Name_File, struct DofData * DofData_P, int Format,
 		      double Val_Time, double Val_TimeImag, int Val_TimeStep) 
 {
-  Dof_OpenFile(DOF_RES, Name_File, (char*)(Format ? "ab" : "a")) ;
+  if(Msg::GetCommRank()) return;
 
+  Dof_OpenFile(DOF_RES, Name_File, (char*)(Format ? "ab" : "a")) ;
+    
   fprintf(File_RES, "$Solution  /* DofData #%d */\n", DofData_P->Num) ;
   fprintf(File_RES, "%d %.16g %.16g %d\n", DofData_P->Num, Val_Time, 
-	  Val_TimeImag, Val_TimeStep) ;
+          Val_TimeImag, Val_TimeStep) ;
 
   Format ? 
     LinAlg_WriteVector(File_RES, &DofData_P->CurrentSolution->x) :
@@ -514,7 +540,20 @@ void Dof_WriteFileRES(char * Name_File, struct DofData * DofData_P, int Format,
 void Dof_WriteFileRES_WithEntityNum(char * Name_File, struct DofData * DofData_P,
                                     struct Group *Group_P, bool saveFixed) 
 {
-  FILE *fp = fopen(Name_File, "w");
+  if(Msg::GetCommRank()) return;
+
+  char    FileCplx[256] ;
+  char    FileRe[256] ;
+  char    FileIm[256] ;
+
+  strcpy(FileCplx, Name_File) ; strcat(FileCplx, ".txt") ;
+  strcpy(FileRe, Name_File) ; strcat(FileRe, "_Re.txt") ;
+  strcpy(FileIm, Name_File) ; strcat(FileIm, "_Im.txt") ;
+
+  FILE *fp   = fopen(FileCplx, "w");
+  FILE *fpRe = fopen(FileRe, "w");
+  FILE *fpIm = fopen(FileIm, "w");
+
   if(!fp){
     Msg::Error("Unable to open file '%s'", Name_File) ;
     return;
@@ -542,13 +581,23 @@ void Dof_WriteFileRES_WithEntityNum(char * Name_File, struct DofData * DofData_P
   }
 
   if(!Group_P){
+    fprintf(fpRe, "%d\n", N);//Needed for ListFromFile
+    fprintf(fpIm, "%d\n", N);
+
     for(std::map<int, std::complex<double> >::iterator it = unknowns.begin();
-        it != unknowns.end(); it++)
-      fprintf(fp, "%d %g %g\n", it->first, it->second.real(), it->second.imag());
+        it != unknowns.end(); it++){
+      fprintf(fp  , "%d %g %g\n", it->first, it->second.real(), it->second.imag());
+      fprintf(fpRe, "%d %g\n", it->first, it->second.real());
+      fprintf(fpIm, "%d %g\n", it->first, it->second.imag());
+    }
   }
   else{
     Msg::Info("Writing solution for all entities in group '%s'", Group_P->Name) ;
     if(!Group_P->ExtendedList) Generate_ExtendedGroup(Group_P) ;
+
+    fprintf(fpRe, "%d\n", List_Nbr(Group_P->ExtendedList));//Needed for ListFromFile
+    fprintf(fpIm, "%d\n", List_Nbr(Group_P->ExtendedList));
+
     for(int i = 0; i < List_Nbr(Group_P->ExtendedList); i++){
       int num;
       List_Read(Group_P->ExtendedList, i, &num);
@@ -558,10 +607,14 @@ void Dof_WriteFileRES_WithEntityNum(char * Name_File, struct DofData * DofData_P
         if(unknowns.count(num)){
           std::complex<double> s = unknowns[num];
           fprintf(fp, "%d %g %g\n", num, s.real(), s.imag());
+          fprintf(fpRe, "%d %g\n", num, s.real());
+          fprintf(fpIm, "%d %g\n", num, s.imag());
         }
         else{
           // yes, write zero: that's on purpose for the iterative schemes
           fprintf(fp, "%d 0 0\n", num);
+          fprintf(fpRe, "%d 0\n", num);
+          fprintf(fpIm, "%d 0\n", num);
         }
       }
     }
@@ -570,6 +623,8 @@ void Dof_WriteFileRES_WithEntityNum(char * Name_File, struct DofData * DofData_P
   if(l) List_Delete(l);
   
   fclose(fp);
+  fclose(fpRe);
+  fclose(fpIm);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -580,6 +635,8 @@ void Dof_ReadFileRES(List_T * DofData_L, struct DofData * Read_DofData_P,
 		     int Read_DofData, double *Time, double *TimeImag,
 		     double *TimeStep) 
 {
+  Msg::Barrier();
+
   int             Num_DofData, Val_TimeStep, Format = 0, Read ;
   double          Val_Time, Val_TimeImag = 0., Version = 0.;
   struct DofData  * DofData_P = NULL ;
@@ -1089,10 +1146,9 @@ void Dof_AssembleInMat(struct Dof * Equ_P, struct Dof * Dof_P, int NbrHar,
       
     case DOF_UNKNOWN :
       if(NbrHar==1){
-	if(Val[0])
-	  LinAlg_AddDoubleInMatrix
-	    (Val[0], Mat, 
-	     Equ_P->Case.Unknown.NumDof-1, Dof_P->Case.Unknown.NumDof-1) ;
+        LinAlg_AddDoubleInMatrix
+          (Val[0], Mat, 
+           Equ_P->Case.Unknown.NumDof-1, Dof_P->Case.Unknown.NumDof-1) ;
       }
       else
 	LinAlg_AddComplexInMatrix
@@ -1673,8 +1729,8 @@ void Dof_GetDummies(struct DefineSystem * DefineSystem_P, struct DofData * DofDa
 	      }
 	    }
 	    Msg::Info("Freq %e (%d/%d) Form %d  Quant %d  Global %d  #dummies %d/%d", 
-		Val_Pulsation[iHar/2]/TWO_PI, iHar/2, Current.NbrHar/2, 
-		i, j, ((struct GlobalQuantity *)GlobalQuantity_P)->Num, ii, iit) ;
+                      Val_Pulsation[iHar/2]/TWO_PI, iHar/2, Current.NbrHar/2, 
+                      i, j, ((struct GlobalQuantity *)GlobalQuantity_P)->Num, ii, iit) ;
 	  }
 	    
 	}   /*  end DummyFrequency in DofData */ 
