@@ -8,14 +8,11 @@
 
 #include "DofData.h"
 
-struct EigenPar {
-  double prec;
-  int    size;
-  int    reortho;
-} ;
-
-void EigenPar(const char *filename, struct EigenPar *par);
 void EigenSolve(struct DofData * DofData_P, int NumEigenvalues, 
 		double shift_r, double shift_i);
+void EigenSolve_ARPACK(struct DofData * DofData_P, int NumEigenvalues, 
+                       double shift_r, double shift_i);
+void EigenSolve_SLEPC(struct DofData * DofData_P, int NumEigenvalues, 
+                      double shift_r, double shift_i);
 
 #endif
