@@ -10,11 +10,20 @@
 //
 
 #include <vector>
+#include <algorithm>
+#include <complex>
+#include <string>
 #include <cstring>
+#include <stdio.h>
 #include "GetDPConfig.h"
 #include "LinAlg.h"
 #include "MallocUtils.h"
 #include "Message.h"
+#include "OS.h"
+
+#if defined(HAVE_SLEPC)
+#include <slepc.h>
+#endif
 
 // Johan, we curse you for a thousand generations!
 #include "ProData.h"
@@ -23,52 +32,43 @@ extern struct CurrentData Current ;
 
 #if defined(HAVE_PETSC)
 
-/* 
-   Options for PETSc can be provided on the command line, or in the file
-   ~/.petscrc. 
+// Options for PETSc can be provided on the command line, or in the file
+// ~/.petscrc. 
+//
+// By default we try to use the MUMPS or UMFPACK direct solver (if
+// available, with PETSc 3). Otherwise we use a GMRES iterative solver
+// preconditionned with an ILU(6).
+//
+// All these options can be changed at runtime. For example you could
+// use
+//
+//   -pc_type ilu
+//   -pc_factor_levels 0
+//   -ksp_rtol 1.e-6
+//   -ksp_gmres_restart 500
+//   -ksp_monitor
+//
+// for GMRES with ILU(0), with a restart of 500 and a stopping
+// criterion of 1e-6. Or you could set
+//
+//   -pc_type lu
+//   -pc_factor_mat_solver_package mumps
+//   -ksp_type gmres
+//
+// to use the MUMPS direct solver as a preconditioner to GMRES (useful
+// e.g if the matrix changed a bit, but you want to keep the same
+// factorization using "SolveAgain").
 
-   By default, we use the following options (GMRES iterative solver,
-   1.e-10 relative tolerance with ILU(6) preconditioner and RCMK
-   renumbering):
-
-     -pc_type ilu
-     -pc_ilu_levels 6 (version 2.3.0) 
-        or -pc_factor_levels 6 (version 2.3.1)
-     -pc_ilu_mat_ordering_type rcm (version 2.3.0)
-        or -pc_factor_mat_ordering rcm (version 2.3.1)
-     -ksp_rtol 1.e-10
-
-   Other useful options include:
-
-     -ksp_gmres_restart 100
-     -ksp_monitor
-     ...
-
-   To use a direct solver (a sparse lu) instead of an iterative
-   solver, use
-
-     -ksp_type preonly -pc_type lu
-
-   When PETSc is compiled with external solvers (UMFPACK, SuperLU,
-   etc.), they can be accessed simply by changing the matrix type. For
-   example, with umfpack:
-
-     -mat_type umfpack (version 2.3.x)
-       or -pc_factor_mat_solver_package umfpack (version 3.0.0)
-*/
-
-#define MYCHECK(ierr) CHKERRABORT(PETSC_COMM_WORLD,ierr)
-static int  ierr, SolverInitialized = 0;
-
-void LinAlg_Initialize(int* argc, char*** argv)
-{
-  MPI_Init(argc, argv);
-}
+static void _try(int ierr){ CHKERRABORT(PETSC_COMM_WORLD, ierr); }
+static int SolverInitialized = 0;
 
 void LinAlg_InitializeSolver(int* argc, char*** argv)
 {
   // This function detects if MPI is initialized
   PetscInitialize(argc, argv, PETSC_NULL, PETSC_NULL);
+#if defined(HAVE_SLEPC)
+  SlepcInitialize(argc, argv, PETSC_NULL, PETSC_NULL);
+#endif
   SolverInitialized = 1;
 
   // get additional petsc options from specified file (useful e.g. on
@@ -84,21 +84,14 @@ void LinAlg_InitializeSolver(int* argc, char*** argv)
   }
 }
 
-void LinAlg_Finalize()
-{
-  int flag;
-  MPI_Initialized(&flag);
-  if(flag) MPI_Finalize();
-}
-
 void LinAlg_FinalizeSolver()
 {
-  if(SolverInitialized) PetscFinalize();
-}
-
-void LinAlg_Barrier()
-{
-  MPI_Barrier(PETSC_COMM_WORLD);
+  if(SolverInitialized){
+#if defined(HAVE_SLEPC)
+    SlepcFinalize();
+#endif
+    PetscFinalize();
+  }
 }
 
 void LinAlg_CreateSolver(gSolver *Solver, const char *SolverDataFileName)
@@ -108,11 +101,33 @@ void LinAlg_CreateSolver(gSolver *Solver, const char *SolverDataFileName)
 
 void LinAlg_CreateVector(gVector *V, gSolver *Solver, int n)
 {
-  ierr = VecCreate(PETSC_COMM_WORLD, &V->V); MYCHECK(ierr);
-  ierr = VecSetSizes(V->V, PETSC_DECIDE, n); MYCHECK(ierr);
+  _try(VecCreate(PETSC_COMM_WORLD, &V->V));
+  _try(VecSetSizes(V->V, PETSC_DECIDE, n));
   // override the default options with the ones from the option
   // database (if any)
-  ierr = VecSetFromOptions(V->V); MYCHECK(ierr);
+  _try(VecSetFromOptions(V->V));
+
+  // create sequential vector that will contain all the values on all
+  // the procs
+  if(Msg::GetCommSize() > 1)
+    _try(VecCreateSeq(PETSC_COMM_SELF, n, &V->Vseq));
+}
+
+static void _fillseq(gVector *V)
+{
+  if(Msg::GetCommSize() == 1) return;
+  // collect all the values from the parallel petsc vector into a
+  // sequential vector on each processor
+  VecScatter ctx;
+  VecScatterCreateToAll(V->V, &ctx, &V->Vseq);
+#if (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR < 3) 
+  VecScatterBegin(V->V, V->Vseq, INSERT_VALUES, SCATTER_FORWARD, ctx);
+  VecScatterEnd(V->V, V->Vseq, INSERT_VALUES, SCATTER_FORWARD, ctx);
+#else
+  VecScatterBegin(ctx, V->V, V->Vseq, INSERT_VALUES, SCATTER_FORWARD);
+  VecScatterEnd(ctx, V->V, V->Vseq, INSERT_VALUES, SCATTER_FORWARD);
+#endif
+  VecScatterDestroy(ctx);
 }
 
 void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
@@ -120,8 +135,11 @@ void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
   PetscInt prealloc = 100;
   PetscTruth set;
   PetscOptionsGetInt(PETSC_NULL, "-petsc_prealloc", &prealloc, &set);
-  std::vector<int> nnz(n, prealloc);
-  
+
+  // prealloc cannot be bigger than the number of rows!
+  prealloc = (n < prealloc) ? n : prealloc; 
+  std::vector<PetscInt> nnz(n, prealloc);
+
   // preallocate non local equations as full lines (this is not
   // optimal, but preallocating too few elements leads to horrible
   // assembly performance: petsc really sucks at dynamic reallocation
@@ -129,30 +147,33 @@ void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
   for(unsigned int i = 0; i < Current.DofData->NonLocalEquations.size(); i++)
     nnz[Current.DofData->NonLocalEquations[i] - 1] = n;
 
-  ierr = MatCreateSeqAIJ(PETSC_COMM_WORLD, n, m, 0, &nnz[0], &M->M);
+  if(Msg::GetCommSize() > 1) // FIXME: alloc full lines...
+    _try(MatCreateMPIAIJ(PETSC_COMM_WORLD, PETSC_DECIDE, PETSC_DECIDE, n, m, 
+                         prealloc, PETSC_NULL, prealloc, PETSC_NULL, &M->M));
+  else
+    _try(MatCreateSeqAIJ(PETSC_COMM_WORLD, n, m, 0, &nnz[0], &M->M));
 
   // override the default options with the ones from the option
   // database (if any)
-  ierr = MatSetFromOptions(M->M); MYCHECK(ierr);
+  _try(MatSetFromOptions(M->M));
 }
 
 void LinAlg_DestroySolver(gSolver *Solver)
 {
-  for(int i = 0; i < 10; i++){
-    if(Solver->ksp[i]){
-      ierr = KSPDestroy(Solver->ksp[i]); MYCHECK(ierr);
-    }
-  }
+  for(int i = 0; i < 10; i++)
+    if(Solver->ksp[i]) _try(KSPDestroy(Solver->ksp[i]));
 }
 
 void LinAlg_DestroyVector(gVector *V)
 {
-  ierr = VecDestroy(V->V); MYCHECK(ierr);
+  _try(VecDestroy(V->V));
+  if(Msg::GetCommSize() > 1)
+    _try(VecDestroy(V->Vseq));
 }
 
 void LinAlg_DestroyMatrix(gMatrix *M)
 {
-  ierr = MatDestroy(M->M); MYCHECK(ierr);
+  _try(MatDestroy(M->M));
 }
 
 void LinAlg_CopyScalar(gScalar *S1, gScalar *S2)
@@ -162,12 +183,14 @@ void LinAlg_CopyScalar(gScalar *S1, gScalar *S2)
 
 void LinAlg_CopyVector(gVector *V1, gVector *V2)
 {
-  ierr = VecCopy(V1->V, V2->V); MYCHECK(ierr);
+  _try(VecCopy(V1->V, V2->V));
+  if(Msg::GetCommSize() > 1)
+    _try(VecCopy(V1->Vseq, V2->Vseq));
 }
 
 void LinAlg_CopyMatrix(gMatrix *M1, gMatrix *M2)
 {
-  ierr = MatCopy(M1->M, M2->M, DIFFERENT_NONZERO_PATTERN); MYCHECK(ierr);
+  _try(MatCopy(M1->M, M2->M, DIFFERENT_NONZERO_PATTERN));
 }
 
 void LinAlg_ZeroScalar(gScalar *S)
@@ -179,12 +202,14 @@ void LinAlg_ZeroVector(gVector *V)
 {
   PetscScalar zero = 0.0;
 
-  ierr = VecSet(V->V, zero); MYCHECK(ierr);
+  _try(VecSet(V->V, zero));
+  if(Msg::GetCommSize() > 1)
+    _try(VecSet(V->Vseq, zero));
 }
 
 void LinAlg_ZeroMatrix(gMatrix *M)
 {
-  ierr = MatZeroEntries(M->M); MYCHECK(ierr);
+  _try(MatZeroEntries(M->M));
 }
 
 void LinAlg_ScanScalar(FILE *file, gScalar *S)
@@ -200,8 +225,10 @@ void LinAlg_ScanScalar(FILE *file, gScalar *S)
 
 void LinAlg_ScanVector(FILE *file, gVector *V)
 {
+  if(Msg::GetCommSize() > 1)
+    Msg::Error("ScanVector not implemented in parallel");
   PetscInt n;  
-  ierr = VecGetSize(V->V, &n); MYCHECK(ierr);
+  _try(VecGetSize(V->V, &n));
   for(PetscInt i = 0; i < n; i++){
     double a, b;
     PetscScalar tmp;
@@ -212,35 +239,36 @@ void LinAlg_ScanVector(FILE *file, gVector *V)
     fscanf(file, "%lf", &a);
     tmp = a;
 #endif
-    ierr = VecSetValues(V->V, 1, &i, &tmp, INSERT_VALUES); MYCHECK(ierr);
+    _try(VecSetValues(V->V, 1, &i, &tmp, INSERT_VALUES));
   }
 }
 
 void LinAlg_ScanMatrix(FILE *file, gMatrix *M)
 {
-  Msg::Error("'LinAlg_ScanMatrix' not yet implemented");  
+  Msg::Error("ScanMatrix not yet implemented");  
 }
 
 void LinAlg_ReadScalar(FILE *file, gScalar *S)
 {
-  Msg::Error("'LinAlg_ReadScalar' not yet implemented");
+  Msg::Error("ReadScalar not yet implemented");
 }
 
 void LinAlg_ReadVector(FILE *file, gVector *V)
 {
+  if(Msg::GetCommSize() > 1)
+    Msg::Error("ReadVector not implemented in parallel");
   PetscInt n;
-  ierr = VecGetSize(V->V, &n); MYCHECK(ierr);
+  _try(VecGetSize(V->V, &n));
   PetscScalar *tmp = (PetscScalar*)Malloc(n*sizeof(PetscScalar));
   fread(tmp, sizeof(PetscScalar), n, file);
-  for(PetscInt i = 0; i < n; i++){
-    ierr = VecSetValues(V->V, 1, &i, &tmp[i], INSERT_VALUES); MYCHECK(ierr);
-  }
+  for(PetscInt i = 0; i < n; i++)
+    _try(VecSetValues(V->V, 1, &i, &tmp[i], INSERT_VALUES));
   Free(tmp);
 }
 
 void LinAlg_ReadMatrix(FILE *file, gMatrix *M)
 {
-  Msg::Error("'LinAlg_ReadMatrix' not yet implemented");  
+  Msg::Error("ReadMatrix not yet implemented");  
 }
 
 void LinAlg_PrintScalar(FILE *file, gScalar *S)
@@ -256,9 +284,10 @@ void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab)
 {
   if(!matlab){
     PetscInt n;
-    ierr = VecGetLocalSize(V->V, &n); MYCHECK(ierr);
+    _try(VecGetSize(V->V, &n));
+    Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
     PetscScalar *tmp;
-    ierr = VecGetArray(V->V, &tmp); MYCHECK(ierr);
+    _try(VecGetArray(VV, &tmp));
     for (int i = 0; i < n; i++){
 #if defined(PETSC_USE_COMPLEX)
       fprintf(file, "%.16g %.16g\n", real(tmp[i]), imag(tmp[i]));
@@ -267,14 +296,14 @@ void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab)
 #endif
     }
     fflush(file);
-    ierr = VecRestoreArray(V->V, &tmp); MYCHECK(ierr);
+    _try(VecRestoreArray(VV, &tmp));
   }
   else{
     PetscViewer fd;
-    PetscViewerASCIIOpen(PETSC_COMM_WORLD, "vector.m", &fd); MYCHECK(ierr);
-    ierr = PetscViewerSetFormat(fd, PETSC_VIEWER_ASCII_MATLAB); MYCHECK(ierr);
-    ierr = VecView(V->V, fd); MYCHECK(ierr);
-    ierr = PetscViewerDestroy(fd); MYCHECK(ierr);
+    _try(PetscViewerASCIIOpen(PETSC_COMM_WORLD, "vector.m", &fd));
+    _try(PetscViewerSetFormat(fd, PETSC_VIEWER_ASCII_MATLAB));
+    _try(VecView(V->V, fd));
+    _try(PetscViewerDestroy(fd));
   }
 } 
 
@@ -282,54 +311,63 @@ void LinAlg_PrintMatrix(FILE *file, gMatrix *M, bool matlab)
 {
   if(!matlab) Msg::Error("Non-matlab output not available for this matrix");
   PetscViewer fd;
-  PetscViewerASCIIOpen(PETSC_COMM_WORLD, "matrix.m", &fd); MYCHECK(ierr);
-  ierr = PetscViewerSetFormat(fd, PETSC_VIEWER_ASCII_MATLAB); MYCHECK(ierr);
-  ierr = MatView(M->M, fd); MYCHECK(ierr);
-  ierr = PetscViewerDestroy(fd); MYCHECK(ierr);
+  _try(PetscViewerASCIIOpen(PETSC_COMM_WORLD, "matrix.m", &fd));
+  _try(PetscViewerSetFormat(fd, PETSC_VIEWER_ASCII_MATLAB));
+  _try(MatView(M->M, fd));
+  _try(PetscViewerDestroy(fd));
 }
 
 void LinAlg_WriteScalar(FILE *file, gScalar *S)
 {
-  Msg::Error("'LinAlg_WriteScalar' not yet implemented");
+  Msg::Error("WriteScalar not yet implemented");
 }
 
 void LinAlg_WriteVector(FILE *file, gVector *V)
 {
   PetscInt n;
-  ierr = VecGetLocalSize(V->V, &n); MYCHECK(ierr);
+  _try(VecGetSize(V->V, &n));
+  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
-  ierr = VecGetArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecGetArray(VV, &tmp));
   fwrite(tmp, sizeof(PetscScalar), n, file);
   fprintf(file, "\n");
-  ierr = VecRestoreArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecRestoreArray(VV, &tmp));
 }
 
 void LinAlg_WriteMatrix(FILE *file, gMatrix *M)
 {
-  Msg::Error("'LinAlg_WriteMatrix' not yet implemented");  
+  Msg::Error("WriteMatrix not yet implemented");  
 }
 
 void LinAlg_GetVectorSize(gVector *V, int *i)
 {
   PetscInt t;
-  ierr = VecGetSize(V->V, &t); MYCHECK(ierr);
-  if(t > INT_MAX) Msg::Error("Problem too big\n");
+  _try(VecGetSize(V->V, &t));
+  if(t > INT_MAX) Msg::Error("Problem too big");
   *i = t;
 }
 
 void LinAlg_GetLocalVectorRange(gVector *V, int *low, int *high)
 {
   PetscInt tlow, thigh;
-  ierr = VecGetOwnershipRange(V->V, &tlow, &thigh); MYCHECK(ierr);
+  _try(VecGetOwnershipRange(V->V, &tlow, &thigh));
   if(tlow > INT_MAX || thigh > INT_MAX) Msg::Error("Problem too big");
   *low = tlow;
   *high = thigh;
 }
 
+static bool _isInLocalRange(gVector *V, int i)
+{
+  if(Msg::GetCommSize() == 1) return true;
+  int imin, imax;
+  LinAlg_GetLocalVectorRange(V, &imin, &imax);
+  return (i >= imin && i < imax);
+}
+
 void LinAlg_GetMatrixSize(gMatrix *M, int *i, int *j)
 {
   PetscInt ti, tj;
-  ierr = MatGetSize(M->M, &ti, &tj); MYCHECK(ierr);
+  _try(MatGetSize(M->M, &ti, &tj));
   if(ti > INT_MAX || tj > INT_MAX) Msg::Error("Problem too big");
   *i = ti;
   *j = tj;
@@ -338,12 +376,20 @@ void LinAlg_GetMatrixSize(gMatrix *M, int *i, int *j)
 void LinAlg_GetLocalMatrixRange(gMatrix *M, int *low, int *high)
 {
   PetscInt tlow, thigh;
-  ierr = MatGetOwnershipRange(M->M, &tlow, &thigh); MYCHECK(ierr);
+  _try(MatGetOwnershipRange(M->M, &tlow, &thigh));
   if(tlow > INT_MAX || thigh > INT_MAX) Msg::Error("Problem too big");
   *low = tlow;
   *high = thigh;
 }
 
+static bool _isInLocalRange(gMatrix *M, int i)
+{
+  if(Msg::GetCommSize() == 1) return true;
+  int imin, imax;
+  LinAlg_GetLocalMatrixRange(M, &imin, &imax);
+  return (i >= imin && i < imax);
+}
+
 void LinAlg_GetDoubleInScalar(double *d, gScalar *S)
 {
 #if defined(PETSC_USE_COMPLEX)
@@ -365,40 +411,44 @@ void LinAlg_GetComplexInScalar(double *d1, double *d2, gScalar *S)
 
 void LinAlg_GetScalarInVector(gScalar *S, gVector *V, int i)
 {
+  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
-  ierr = VecGetArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecGetArray(VV, &tmp));
   S->s = tmp[i];
-  ierr = VecRestoreArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecRestoreArray(VV, &tmp));
 }
 
 void LinAlg_GetDoubleInVector(double *d, gVector *V, int i)
 {
+  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
-  ierr = VecGetArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecGetArray(VV, &tmp));
 #if defined(PETSC_USE_COMPLEX)
   *d = real(tmp[i]);
 #else
   *d = tmp[i];
 #endif
-  ierr = VecRestoreArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecRestoreArray(VV, &tmp));
 }
 
 void LinAlg_GetAbsDoubleInVector(double *d, gVector *V, int i)
 {
+  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
-  ierr = VecGetArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecGetArray(VV, &tmp));
 #if defined(PETSC_USE_COMPLEX)
   *d = fabs(real(tmp[i]));
 #else
   *d = fabs(tmp[i]);
 #endif
-  ierr = VecRestoreArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecRestoreArray(VV, &tmp));
 }
 
 void LinAlg_GetComplexInVector(double *d1, double *d2, gVector *V, int i, int j)
 {
+  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
-  ierr = VecGetArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecGetArray(VV, &tmp));
 #if defined(PETSC_USE_COMPLEX)
   *d1 = real(tmp[i]);
   *d2 = imag(tmp[i]);
@@ -406,27 +456,27 @@ void LinAlg_GetComplexInVector(double *d1, double *d2, gVector *V, int i, int j)
   *d1 = (double)tmp[i];
   *d2 = (double)tmp[j];
 #endif
-  ierr = VecRestoreArray(V->V, &tmp); MYCHECK(ierr);
+  _try(VecRestoreArray(VV, &tmp));
 }
 
 void LinAlg_GetScalarInMatrix(gScalar *S, gMatrix *M, int i, int j)
 {
-  Msg::Error("'LinAlg_GetScalarInMatrix' not yet implemented");  
+  Msg::Error("GetScalarInMatrix not yet implemented");  
 }
 
 void LinAlg_GetDoubleInMatrix(double *d, gMatrix *M, int i, int j)
 {
-  Msg::Error("'LinAlg_GetDoubleInMatrix' not yet implemented");  
+  Msg::Error("GetDoubleInMatrix not yet implemented");  
 }
 
 void LinAlg_GetComplexInMatrix(double *d1, double *d2, gMatrix *M, int i, int j, int k, int l)
 {
-  Msg::Error("'LinAlg_GetComplexInMatrix' not yet implemented");  
+  Msg::Error("GetComplexInMatrix not yet implemented");  
 }
 
 void LinAlg_GetColumnInMatrix(gMatrix *M, int col, gVector *V1)
 {
-  Msg::Error("'LinAlg_GetColumnInMatrix' not yet implemented");  
+  Msg::Error("GetColumnInMatrix not yet implemented");  
 }
 
 void LinAlg_SetScalar(gScalar *S, double *d)
@@ -441,52 +491,63 @@ void LinAlg_SetScalar(gScalar *S, double *d)
 void LinAlg_SetVector(gVector *V, double *v)
 {
   PetscScalar tmp = *v;
-  ierr = VecSet(V->V, tmp); MYCHECK(ierr);
+  _try(VecSet(V->V, tmp));
+  if(Msg::GetCommSize() > 1)
+    _try(VecSet(V->Vseq, tmp));
 }
 
 void LinAlg_SetScalarInVector(gScalar *S, gVector *V, int i)
 {
-  PetscInt t = i;
-  ierr = VecSetValues(V->V, 1, &t, &S->s, INSERT_VALUES); MYCHECK(ierr);
+  if(!_isInLocalRange(V, i)) return;
+  PetscInt ti = i;
+  _try(VecSetValues(V->V, 1, &ti, &S->s, INSERT_VALUES));
 }
 
 void LinAlg_SetDoubleInVector(double d, gVector *V, int i)
 {
+  if(!_isInLocalRange(V, i)) return;
   PetscScalar tmp = d;
-  PetscInt t = i;
-  ierr = VecSetValues(V->V, 1, &t, &tmp, INSERT_VALUES); MYCHECK(ierr);
+  PetscInt ti = i;
+  _try(VecSetValues(V->V, 1, &ti, &tmp, INSERT_VALUES));
 }
 
 void LinAlg_SetComplexInVector(double d1, double d2, gVector *V, int i, int j)
 {
   PetscScalar tmp;
 #if defined(PETSC_USE_COMPLEX)
-  PetscInt ti = i;
-  tmp = d1 + PETSC_i * d2;
-  ierr = VecSetValues(V->V, 1, &ti, &tmp, INSERT_VALUES); MYCHECK(ierr);
+  if(_isInLocalRange(V, i)){
+    PetscInt ti = i;
+    tmp = d1 + PETSC_i * d2;
+    _try(VecSetValues(V->V, 1, &ti, &tmp, INSERT_VALUES));
+  }
 #else
   PetscInt ti = i, tj = j;
-  tmp = d1;
-  ierr = VecSetValues(V->V, 1, &ti, &tmp, INSERT_VALUES); MYCHECK(ierr);
-  tmp = d2;
-  ierr = VecSetValues(V->V, 1, &tj, &tmp, INSERT_VALUES); MYCHECK(ierr);
+  if(_isInLocalRange(V, i)){
+    tmp = d1;
+    _try(VecSetValues(V->V, 1, &ti, &tmp, INSERT_VALUES));
+  }
+  if(_isInLocalRange(V, j)){
+    tmp = d2;
+    _try(VecSetValues(V->V, 1, &tj, &tmp, INSERT_VALUES));
+  }
 #endif
 }
 
 void LinAlg_SetScalarInMatrix(gScalar *S, gMatrix *M, int i, int j)
 {
+  if(!_isInLocalRange(M, i)) return;
   PetscInt ti = i, tj = j;
-  ierr = MatSetValues(M->M, 1, &ti, 1, &tj, &S->s, INSERT_VALUES); MYCHECK(ierr);
+  _try(MatSetValues(M->M, 1, &ti, 1, &tj, &S->s, INSERT_VALUES));
 }
 
 void LinAlg_SetDoubleInMatrix(double d, gMatrix *M, int i, int j)
 {
-  Msg::Error("'LinAlg_SetDoubleInMatrix' not yet implemented");  
+  Msg::Error("SetDoubleInMatrix not yet implemented");  
 }
 
 void LinAlg_SetComplexInMatrix(double d1, double d2, gMatrix *M, int i, int j, int k, int l)
 {
-  Msg::Error("'LinAlg_SetComplexInMatrix' not yet implemented");  
+  Msg::Error("SetComplexInMatrix not yet implemented");  
 }
 
 void LinAlg_AddScalarScalar(gScalar *S1, gScalar *S2, gScalar *S3)
@@ -497,50 +558,60 @@ void LinAlg_AddScalarScalar(gScalar *S1, gScalar *S2, gScalar *S3)
 void LinAlg_DummyVector(gVector *V)
 {
   if(Current.DofData->DummyDof)
-    Msg::Error("'LinAlg_DummyVector' not yet implemented");
+    Msg::Error("DummyVector not yet implemented");
   return;
 }
 
 void LinAlg_AddScalarInVector(gScalar *S, gVector *V, int i)
 {
+  if(!_isInLocalRange(V, i)) return;
   PetscInt ti = i;
-  ierr = VecSetValues(V->V, 1, &ti, &S->s, ADD_VALUES); MYCHECK(ierr);
+  _try(VecSetValues(V->V, 1, &ti, &S->s, ADD_VALUES));
 }
 
 void LinAlg_AddDoubleInVector(double d, gVector *V, int i)
 {
+  if(!_isInLocalRange(V, i)) return;
   PetscScalar tmp = d;
-  PetscInt t = i;
-  ierr = VecSetValues(V->V, 1, &t, &tmp, ADD_VALUES); MYCHECK(ierr);
+  PetscInt ti = i;
+  _try(VecSetValues(V->V, 1, &ti, &tmp, ADD_VALUES));
 }
 
 void LinAlg_AddComplexInVector(double d1, double d2, gVector *V, int i, int j)
 {
   PetscScalar tmp;
 #if defined(PETSC_USE_COMPLEX)
-  PetscInt ti = i;
-  tmp = d1 + PETSC_i * d2;
-  ierr = VecSetValues(V->V, 1, &ti, &tmp, ADD_VALUES); MYCHECK(ierr);
+  if(_isInLocalRange(V, i)){
+    PetscInt ti = i;
+    tmp = d1 + PETSC_i * d2;
+    _try(VecSetValues(V->V, 1, &ti, &tmp, ADD_VALUES));
+  }
 #else
   PetscInt ti = i, tj = j;
-  tmp = d1;
-  ierr = VecSetValues(V->V, 1, &ti, &tmp, ADD_VALUES); MYCHECK(ierr);
-  tmp = d2;
-  ierr = VecSetValues(V->V, 1, &tj, &tmp, ADD_VALUES); MYCHECK(ierr);
+  if(_isInLocalRange(V, i)){
+    tmp = d1;
+    _try(VecSetValues(V->V, 1, &ti, &tmp, ADD_VALUES));
+  }
+  if(_isInLocalRange(V, j)){
+    tmp = d2;
+    _try(VecSetValues(V->V, 1, &tj, &tmp, ADD_VALUES));
+  }
 #endif
 }
 
 void LinAlg_AddScalarInMatrix(gScalar *S, gMatrix *M, int i, int j)
 {
+  if(!_isInLocalRange(M, i)) return;
   PetscInt ti = i, tj = j;
-  ierr = MatSetValues(M->M, 1, &ti, 1, &tj, &S->s, ADD_VALUES); MYCHECK(ierr);
+  _try(MatSetValues(M->M, 1, &ti, 1, &tj, &S->s, ADD_VALUES));
 }
 
 void LinAlg_AddDoubleInMatrix(double d, gMatrix *M, int i, int j)
 {
+  if(!_isInLocalRange(M, i)) return;
   PetscScalar tmp = d;
   PetscInt ti = i, tj = j;
-  ierr = MatSetValues(M->M, 1, &ti, 1, &tj, &tmp, ADD_VALUES); MYCHECK(ierr);
+  _try(MatSetValues(M->M, 1, &ti, 1, &tj, &tmp, ADD_VALUES));
 }
 
 void LinAlg_AddComplexInMatrix(double d1, double d2, gMatrix *M, int i, int j, int k, int l)
@@ -548,20 +619,28 @@ void LinAlg_AddComplexInMatrix(double d1, double d2, gMatrix *M, int i, int j, i
   PetscScalar tmp;
 #if defined(PETSC_USE_COMPLEX)
   PetscInt ti = i, tj = j;
-  tmp = d1 + PETSC_i * d2;
-  ierr = MatSetValues(M->M, 1, &ti, 1, &tj, &tmp, ADD_VALUES); MYCHECK(ierr);
+  if(_isInLocalRange(M, i)){
+    tmp = d1 + PETSC_i * d2;
+    _try(MatSetValues(M->M, 1, &ti, 1, &tj, &tmp, ADD_VALUES));
+  }
 #else
   PetscInt ti = i, tj = j, tk = k, tl = l;
   if(d1){
     tmp = d1;
-    ierr = MatSetValues(M->M, 1, &ti, 1, &tj, &tmp, ADD_VALUES); MYCHECK(ierr);
-    ierr = MatSetValues(M->M, 1, &tk, 1, &tl, &tmp, ADD_VALUES); MYCHECK(ierr);
+    if(_isInLocalRange(M, i))
+      _try(MatSetValues(M->M, 1, &ti, 1, &tj, &tmp, ADD_VALUES));
+    if(_isInLocalRange(M, k))
+      _try(MatSetValues(M->M, 1, &tk, 1, &tl, &tmp, ADD_VALUES));
   }
   if(d2){
-    tmp = -d2;
-    ierr = MatSetValues(M->M, 1, &ti, 1, &tl, &tmp, ADD_VALUES); MYCHECK(ierr);
-    tmp = d2;
-    ierr = MatSetValues(M->M, 1, &tk, 1, &tj, &tmp, ADD_VALUES); MYCHECK(ierr);
+    if(_isInLocalRange(M, i)){
+      tmp = -d2;
+      _try(MatSetValues(M->M, 1, &ti, 1, &tl, &tmp, ADD_VALUES));
+    }
+    if(_isInLocalRange(M, k)){
+      tmp = d2;
+      _try(MatSetValues(M->M, 1, &tk, 1, &tj, &tmp, ADD_VALUES));
+    }
   }
 #endif
 }
@@ -570,10 +649,12 @@ void LinAlg_AddVectorVector(gVector *V1, gVector *V2, gVector *V3)
 {
   PetscScalar tmp = 1.0;
   if(V3 == V1){
-    ierr = VecAXPY(V1->V, tmp, V2->V); MYCHECK(ierr);
+    _try(VecAXPY(V1->V, tmp, V2->V));
+    _fillseq(V1);
   }
   else if(V3 == V2){
-    ierr = VecAXPY(V2->V, tmp, V1->V); MYCHECK(ierr);
+    _try(VecAXPY(V2->V, tmp, V1->V));
+    _fillseq(V2);
   }
   else
     Msg::Error("Wrong arguments in 'LinAlg_AddVectorVector'");  
@@ -583,10 +664,12 @@ void LinAlg_AddVectorProdVectorDouble(gVector *V1, gVector *V2, double d, gVecto
 {
   PetscScalar tmp = d;
   if(V3 == V1){
-    ierr = VecAXPY(V1->V, tmp, V2->V); MYCHECK(ierr);
+    _try(VecAXPY(V1->V, tmp, V2->V));
+    _fillseq(V1);
   }
   else if(V3 == V2){
-    ierr = VecAYPX(V2->V, tmp, V1->V); MYCHECK(ierr);
+    _try(VecAYPX(V2->V, tmp, V1->V));
+    _fillseq(V2);
   }
   else
     Msg::Error("Wrong arguments in 'LinAlg_AddVectorProdVectorDouble'");  
@@ -595,12 +678,10 @@ void LinAlg_AddVectorProdVectorDouble(gVector *V1, gVector *V2, double d, gVecto
 void LinAlg_AddMatrixMatrix(gMatrix *M1, gMatrix *M2, gMatrix *M3)
 {
   PetscScalar tmp = 1.0;
-  if(M3 == M1){
-    ierr = MatAXPY(M1->M, tmp, M2->M, DIFFERENT_NONZERO_PATTERN); MYCHECK(ierr);
-  }
-  else if(M3 == M2){
-    ierr = MatAXPY(M2->M, tmp, M1->M, DIFFERENT_NONZERO_PATTERN); MYCHECK(ierr);
-  }
+  if(M3 == M1)
+    _try(MatAXPY(M1->M, tmp, M2->M, DIFFERENT_NONZERO_PATTERN));
+  else if(M3 == M2)
+    _try(MatAXPY(M2->M, tmp, M1->M, DIFFERENT_NONZERO_PATTERN));
   else
     Msg::Error("Wrong arguments in 'LinAlg_AddMatrixMatrix'");  
 }
@@ -608,16 +689,14 @@ void LinAlg_AddMatrixMatrix(gMatrix *M1, gMatrix *M2, gMatrix *M3)
 void LinAlg_AddMatrixProdMatrixDouble(gMatrix *M1, gMatrix *M2, double d, gMatrix *M3)
 {
   PetscScalar tmp = d;
-  if(M3 == M1){
-    ierr = MatAXPY(M1->M, tmp, M2->M, DIFFERENT_NONZERO_PATTERN); MYCHECK(ierr);
-  }
-  else if(M3 == M2){
+  if(M3 == M1)
+    _try(MatAXPY(M1->M, tmp, M2->M, DIFFERENT_NONZERO_PATTERN));
+  else if(M3 == M2)
 #if (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR < 2)
-    ierr = MatAYPX(M2->M, tmp, M1->M); MYCHECK(ierr);
+    _try(MatAYPX(M2->M, tmp, M1->M));
 #else
-    ierr = MatAYPX(M2->M, tmp, M1->M, DIFFERENT_NONZERO_PATTERN); MYCHECK(ierr);
+    _try(MatAYPX(M2->M, tmp, M1->M, DIFFERENT_NONZERO_PATTERN));
 #endif
-  }
   else
     Msg::Error("Wrong arguments in 'LinAlg_AddMatrixProdMatrixDouble'");
 }
@@ -631,10 +710,12 @@ void LinAlg_SubVectorVector(gVector *V1, gVector *V2, gVector *V3)
 {
   PetscScalar tmp = -1.0;
   if(V3 == V1){
-    ierr = VecAXPY(V1->V, tmp, V2->V); MYCHECK(ierr);
+    _try(VecAXPY(V1->V, tmp, V2->V));
+    _fillseq(V1);
   }
   else if(V3 == V2){
-    ierr = VecAYPX(V2->V, tmp, V1->V); MYCHECK(ierr);
+    _try(VecAYPX(V2->V, tmp, V1->V));
+    _fillseq(V2);
   }
   else
     Msg::Error("Wrong arguments in 'LinAlg_SubVectorVector'");  
@@ -642,7 +723,7 @@ void LinAlg_SubVectorVector(gVector *V1, gVector *V2, gVector *V3)
 
 void LinAlg_SubMatrixMatrix(gMatrix *M1, gMatrix *M2, gMatrix *M3)
 {
-  Msg::Error("'LinAlg_SubMatrixMatrix' not yet implemented");    
+  Msg::Error("SubMatrixMatrix not yet implemented");    
 }
 
 void LinAlg_ProdScalarScalar(gScalar *S1, gScalar *S2, gScalar *S3)
@@ -674,7 +755,8 @@ void LinAlg_ProdScalarComplex(gScalar *S, double d1, double d2, double *d3, doub
 void LinAlg_ProdVectorScalar(gVector *V1, gScalar *S, gVector *V2)
 { 
   if(V2 == V1){
-    ierr = VecScale(V1->V, S->s); MYCHECK(ierr);
+    _try(VecScale(V1->V, S->s));
+    _fillseq(V1);
   }
   else
     Msg::Error("Wrong arguments in 'LinAlg_ProdVectorScalar'");
@@ -684,7 +766,8 @@ void LinAlg_ProdVectorDouble(gVector *V1, double d, gVector *V2)
 {
   PetscScalar tmp = d;
   if(V2 == V1){
-    ierr = VecScale(V1->V, tmp); MYCHECK(ierr);
+    _try(VecScale(V1->V, tmp));
+    _fillseq(V1);
   }
   else
     Msg::Error("Wrong arguments in 'LinAlg_ProdVectorDouble'");
@@ -692,13 +775,13 @@ void LinAlg_ProdVectorDouble(gVector *V1, double d, gVector *V2)
 
 void LinAlg_ProdVectorComplex(gVector *V1, double d1, double d2, gVector *V2)
 {
-  Msg::Error("'LinAlg_ProdVectorComplex' not yet implemented");
+  Msg::Error("ProdVectorComplex not yet implemented");
 }
 
 void LinAlg_ProdVectorVector(gVector *V1, gVector *V2, double *d)
 {
   PetscScalar tmp;
-  ierr = VecDot(V1->V, V2->V, &tmp); MYCHECK(ierr);
+  _try(VecDot(V1->V, V2->V, &tmp));
 #if defined(PETSC_USE_COMPLEX)
   *d = real(tmp);
 #else
@@ -710,15 +793,16 @@ void LinAlg_ProdMatrixVector(gMatrix *M, gVector *V1, gVector *V2)
 {
   if(V2 == V1)
     Msg::Error("Wrong arguments in 'LinAlg_ProdMatrixVector'");
-  else
-    ierr = MatMult(M->M, V1->V, V2->V); MYCHECK(ierr);
+  else{
+    _try(MatMult(M->M, V1->V, V2->V));
+    _fillseq(V2);
+  }
 }
 
 void LinAlg_ProdMatrixScalar(gMatrix *M1, gScalar *S, gMatrix *M2)
 {
-  if(M2 == M1){
-    ierr = MatScale(M1->M, S->s); MYCHECK(ierr);
-  }
+  if(M2 == M1)
+    _try(MatScale(M1->M, S->s));
   else
     Msg::Error("Wrong arguments in 'LinAlg_ProdMatrixScalar'");
 }
@@ -726,9 +810,8 @@ void LinAlg_ProdMatrixScalar(gMatrix *M1, gScalar *S, gMatrix *M2)
 void LinAlg_ProdMatrixDouble(gMatrix *M1, double d, gMatrix *M2)
 {
   PetscScalar tmp = d;
-  if(M2 == M1){
-    ierr = MatScale(M1->M, tmp); MYCHECK(ierr);
-  }
+  if(M2 == M1)
+    _try(MatScale(M1->M, tmp));
   else
     Msg::Error("Wrong arguments in 'LinAlg_ProdMatrixDouble'");
 }
@@ -738,12 +821,12 @@ void LinAlg_ProdMatrixComplex(gMatrix *M1, double d1, double d2, gMatrix *M2)
 #if defined(PETSC_USE_COMPLEX)
   if(M2 == M1){
     PetscScalar tmp = d1 + (PETSC_i * d2);
-    ierr = MatScale(M1->M, tmp); MYCHECK(ierr);
+    _try(MatScale(M1->M, tmp));
   }
   else
     Msg::Error("Wrong arguments in 'LinAlg_ProdMatrixDouble'");
 #else
-  Msg::Error("'LinAlg_ProdMatrixComplex' not yet implemented");
+  Msg::Error("ProdMatrixComplex not yet implemented");
 #endif
 }
 
@@ -760,27 +843,30 @@ void LinAlg_DivScalarDouble(gScalar *S1, double d, gScalar *S2)
 void LinAlg_VectorNorm2(gVector *V1, double *norm)
 {
   PetscReal tmp;
-  VecNorm(V1->V, NORM_2, &tmp);
+  _try(VecNorm(V1->V, NORM_2, &tmp));
   *norm = tmp;
 }
 
 void LinAlg_VectorNormInf(gVector *V1, double *norm)
 {
   PetscReal tmp;
-  VecNorm(V1->V, NORM_INFINITY, &tmp);
+  _try(VecNorm(V1->V, NORM_INFINITY, &tmp));
   *norm = tmp;
 }
 
 void LinAlg_AssembleMatrix(gMatrix *M)
 {
-  ierr = MatAssemblyBegin(M->M, MAT_FINAL_ASSEMBLY); MYCHECK(ierr);
-  ierr = MatAssemblyEnd(M->M, MAT_FINAL_ASSEMBLY); MYCHECK(ierr);  
+  Msg::Barrier();
+  _try(MatAssemblyBegin(M->M, MAT_FINAL_ASSEMBLY));
+  _try(MatAssemblyEnd(M->M, MAT_FINAL_ASSEMBLY));
 }
 
 void LinAlg_AssembleVector(gVector *V)
 {
-  ierr = VecAssemblyBegin(V->V); MYCHECK(ierr);
-  ierr = VecAssemblyEnd(V->V); MYCHECK(ierr);
+  Msg::Barrier();
+  _try(VecAssemblyBegin(V->V));
+  _try(VecAssemblyEnd(V->V));
+  _fillseq(V);
 }
 
 #if defined(HAVE_ZITSOL)
@@ -804,9 +890,11 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
   PetscOptionsGetReal(PETSC_NULL, "-zitsol_tol", &tol, &set);
 
   MatInfo info;
-  ierr = MatGetInfo(A->M, MAT_LOCAL, &info);
-  int n = info.rows_local;
+  _try(MatGetInfo(A->M, MAT_LOCAL, &info));
   int nnz = info.nz_used;
+  //int n = info.rows_local;
+  PetscInt n;
+  _try(VecGetLocalSize(B->V, &n));
 
   int *row = (int*)Malloc(nnz * sizeof(int));
   int *col = (int*)Malloc(nnz * sizeof(int));
@@ -822,7 +910,7 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
     PetscInt ncols;
     const PetscInt *cols;
     const PetscScalar *vals;
-    ierr = MatGetRow(A->M, i, &ncols, &cols, &vals);
+    _try(MatGetRow(A->M, i, &ncols, &cols, &vals));
     for(int j = 0; j < ncols; j++){
       if(k >= nnz){
 	Msg::Error("Something wrong in nnz: %d >= %d", k, nnz);
@@ -842,14 +930,14 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
 #endif
       k++;
     }
-    ierr = MatRestoreRow(A->M, i, &ncols, &cols, &vals);
+    _try(MatRestoreRow(A->M, i, &ncols, &cols, &vals));
   }
 
   Msg::Info("n = %d, nnz = %d (check k = %d)", n, nnz, k);
 
   PetscScalar *b, *x;
-  ierr = VecGetArray(B->V, &b); MYCHECK(ierr);
-  ierr = VecGetArray(X->V, &x); MYCHECK(ierr);
+  _try(VecGetArray(B->V, &b));
+  _try(VecGetArray(X->V, &x));
   for(int i = 0; i < n; i++){
 #if defined(PETSC_USE_COMPLEX)
     rhsr[i] = real(b[i]);
@@ -863,8 +951,8 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
     soli[i] = 0.;
 #endif
   }
-  ierr = VecRestoreArray(B->V, &b); MYCHECK(ierr);
-  ierr = VecRestoreArray(X->V, &x); MYCHECK(ierr);
+  _try(VecRestoreArray(B->V, &b));
+  _try(VecRestoreArray(X->V, &x));
 
   int its = getdp_zitsol(n, nnz, row, col, valr, vali, rhsr, rhsi, solr, soli,
                          precond, lfil, tol0, tol, im, maxits);
@@ -880,7 +968,7 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
 #else
     d = solr[i];
 #endif
-    ierr = VecSetValues(X->V, 1, &i, &d, INSERT_VALUES); MYCHECK(ierr);
+    _try(VecSetValues(X->V, 1, &i, &d, INSERT_VALUES));
   }
   
   Free(row); Free(col);
@@ -902,7 +990,7 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
 {
 #if defined(HAVE_ZITSOL)
   // testing Yousef's new preconditioners and solvers
-  PetscTruth zitsol = PETSC_FALSE, set;
+  PetscTruth set, zitsol = PETSC_FALSE;
   PetscOptionsGetTruth(PETSC_NULL, "-zitsol", &zitsol, &set);
   if(zitsol){ _zitsol(A, B, X); return; }
 #endif
@@ -912,60 +1000,73 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
     return;
   }
 
-  int RankCpu;
-  MPI_Comm_rank(PETSC_COMM_WORLD, &RankCpu);
+  PetscInt i, j;
+  _try(MatGetSize(A->M, &i, &j));
+  if(!i){
+    Msg::Warning("Zero-size system: skipping solve!");
+    return;
+  }
 
   bool view = (!Solver->ksp[kspIndex] && Msg::GetVerbosity() > 2);
 
-  if(view && !RankCpu){
-    PetscInt i, j;
-    ierr = MatGetSize(A->M, &i, &j); MYCHECK(ierr);
+  if(view && !Msg::GetCommRank())
     Msg::Info("N: %ld", (long)i);
-  }
 
-  if(kspIndex != 0){
+  if(kspIndex != 0)
     Msg::Info("Using solver index %d", kspIndex);
-  }
 
   if(!Solver->ksp[kspIndex]) {
-    ierr = KSPCreate(PETSC_COMM_WORLD, &Solver->ksp[kspIndex]); MYCHECK(ierr);
-    ierr = KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M, DIFFERENT_NONZERO_PATTERN); MYCHECK(ierr);
+    _try(KSPCreate(PETSC_COMM_WORLD, &Solver->ksp[kspIndex]));
+    _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M, DIFFERENT_NONZERO_PATTERN));
+    if(Msg::UseSocket())
+      _try(KSPMonitorSet(Solver->ksp[kspIndex], _myKspMonitor, PETSC_NULL, PETSC_NULL));
     PC pc;
-    ierr = KSPGetPC(Solver->ksp[kspIndex], &pc); MYCHECK(ierr);
+    _try(KSPGetPC(Solver->ksp[kspIndex], &pc));
+
     // set some default options
-    ierr = PCSetType(pc, PCILU); MYCHECK(ierr);
+    _try(KSPSetTolerances(Solver->ksp[kspIndex], 1.e-12, PETSC_DEFAULT, PETSC_DEFAULT, 
+                          PETSC_DEFAULT));
+#if (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_MUMPS) // use MUMPS by default if available
+    _try(PCSetType(pc, PCLU));
+    _try(PCFactorSetMatSolverPackage(pc, "mumps"));
+    _try(KSPSetType(Solver->ksp[kspIndex], "preonly"));
+#elif (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_UMFPACK) // otherwise use UMFPACK if available
+    _try(PCSetType(pc, PCLU));
+    _try(PCFactorSetMatSolverPackage(pc, "umfpack"));
+    _try(KSPSetType(Solver->ksp[kspIndex], "preonly"));
+#else // otherwise use ILU(6) + GMRES
+    _try(PCSetType(pc, PCILU));
 #if (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR == 0)
-    ierr = PCILUSetMatOrdering(pc, MATORDERING_RCM); MYCHECK(ierr);
-    ierr = PCILUSetLevels(pc, 6); MYCHECK(ierr);
+    _try(PCILUSetMatOrdering(pc, MATORDERING_RCM));
+    _try(PCILUSetLevels(pc, 6));
 #elif (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR < 3)
-    ierr = PCFactorSetMatOrdering(pc, MATORDERING_RCM); MYCHECK(ierr);
-    ierr = PCFactorSetLevels(pc, 6); MYCHECK(ierr);
+    _try(PCFactorSetMatOrdering(pc, MATORDERING_RCM));
+    _try(PCFactorSetLevels(pc, 6));
 #else
-    ierr = PCFactorSetMatOrderingType(pc, MATORDERING_RCM); MYCHECK(ierr);
-    ierr = PCFactorSetLevels(pc, 6); MYCHECK(ierr);
+    _try(PCFactorSetMatOrderingType(pc, MATORDERING_RCM));
+    _try(PCFactorSetLevels(pc, 6));
 #endif
-    ierr = KSPSetTolerances(Solver->ksp[kspIndex], 1.e-10, PETSC_DEFAULT, PETSC_DEFAULT, 
-			    PETSC_DEFAULT); MYCHECK(ierr);
-    if(Msg::UseSocket()){
-      ierr = KSPMonitorSet(Solver->ksp[kspIndex], _myKspMonitor, PETSC_NULL, PETSC_NULL); MYCHECK(ierr);
-    }
+#endif
+
     // override the default options with the ones from the option
     // database (if any)
-    ierr = KSPSetFromOptions(Solver->ksp[kspIndex]); MYCHECK(ierr);
+    _try(KSPSetFromOptions(Solver->ksp[kspIndex]));
   }
   else if(precond){
-    ierr = KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M, DIFFERENT_NONZERO_PATTERN); MYCHECK(ierr);
+    _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M, DIFFERENT_NONZERO_PATTERN));
   }
-  
-  ierr = KSPSolve(Solver->ksp[kspIndex], B->V, X->V); MYCHECK(ierr);
 
-  if(view){
-    ierr = KSPView(Solver->ksp[kspIndex], PETSC_VIEWER_STDOUT_SELF); MYCHECK(ierr);
-  }
+  _try(KSPSolve(Solver->ksp[kspIndex], B->V, X->V));
+
+  // copy result on all procs
+  _fillseq(X);
+
+  if(view)
+    _try(KSPView(Solver->ksp[kspIndex], PETSC_VIEWER_STDOUT_WORLD));
   
-  if(!RankCpu){
+  if(!Msg::GetCommRank()){
     PetscInt its;
-    ierr = KSPGetIterationNumber(Solver->ksp[kspIndex], &its); MYCHECK(ierr);
+    _try(KSPGetIterationNumber(Solver->ksp[kspIndex], &its));
     Msg::Info("%d iterations", its);
   }
 }
