@@ -28,7 +28,7 @@
 extern struct Problem Problem_S ;
 extern struct CurrentData Current ;
 
-extern int    Flag_BIN ;
+extern int    Flag_BIN, Flag_GMSH_VERSION;
 
 extern FILE   *PostStream ;
 
@@ -48,11 +48,13 @@ static List_T *SP = NULL, *VP = NULL, *TP = NULL;
 static List_T *SL = NULL, *VL = NULL, *TL = NULL;
 static List_T *ST = NULL, *VT = NULL, *TT = NULL;
 static List_T *SQ = NULL, *VQ = NULL, *TQ = NULL;
-static List_T *SS = NULL, *VS = NULL, *TS = NULL;
+static List_T *SS = NULL, *VS = NULL, *TS1 = NULL; // TS<->TS1 fix compile with PETSc
 static List_T *SH = NULL, *VH = NULL, *TH = NULL;
 static List_T *SI = NULL, *VI = NULL, *TI = NULL;
 static List_T *SY = NULL, *VY = NULL, *TY = NULL;
 static List_T *T2D = NULL, *T2C = NULL;
+static char    CurrentName[256] = "";
+static int     CurrentPartitionNumber = 0;
 
 /* ------------------------------------------------------------------------ */
 /*  F o r m a t _ P o s t F o r m a t / H e a d e r / F o o t e r           */
@@ -62,7 +64,17 @@ void  Format_PostFormat(int Format)
 {
   switch(Format){
   case FORMAT_GMSH :
-    if(Flag_BIN){/* bricolage */
+    if(Flag_GMSH_VERSION == 2){
+      fprintf(PostStream, "$MeshFormat\n") ;
+      fprintf(PostStream, "2.2 %d %d\n", Flag_BIN, (int)sizeof(double)) ;
+      if(Flag_BIN){
+        int one=1;
+        fwrite(&one, sizeof(int), 1, PostStream);
+        fprintf(PostStream, "\n");
+      }
+      fprintf(PostStream, "$EndMeshFormat\n") ;
+    }
+    else if(Flag_BIN){/* bricolage */
       fprintf(PostStream, "$PostFormat /* Gmsh 1.2, %s */\n",
 	      Flag_BIN ? "binary" : "ascii") ;
       fprintf(PostStream, "1.2 %d %d\n", Flag_BIN, (int)sizeof(double)) ;
@@ -83,6 +95,8 @@ void  Format_PostHeader(int Format, int Contour,
 {
   char name[256] ;
 
+  CurrentPartitionNumber = 0;
+
   if(Contour){
     if(!PostElement_L) 
       PostElement_L = List_Create(20, 20, sizeof(struct PostElement*));
@@ -102,18 +116,22 @@ void  Format_PostHeader(int Format, int Contour,
   else
     strcpy(name, "unnamed");
 
+  strcpy(CurrentName, name);
+
   switch(Format){
   case FORMAT_GMSH_PARSED :
     fprintf(PostStream, "View \"%s\" {\n", name) ;
     Gmsh_StartNewView = 1 ;
     break ;
   case FORMAT_GMSH :
-    if(Flag_BIN){ /* bricolage */
-      fprintf(PostStream, "$View /* %s */\n", name);
-      fprintf(PostStream, "%s ", name);
-    }
-    else{
-      fprintf(PostStream, "View \"%s\" {\n", name) ;
+    if(Flag_GMSH_VERSION != 2){
+      if(Flag_BIN){ /* bricolage */
+        fprintf(PostStream, "$View /* %s */\n", name);
+        fprintf(PostStream, "%s ", name);
+      }
+      else {
+        fprintf(PostStream, "View \"%s\" {\n", name) ;
+      }
     }
     Gmsh_StartNewView = 1 ;
     break ;
@@ -141,11 +159,15 @@ void  Format_PostHeader(int Format, int Contour,
     fprintf(PostStream, "Dr. Binde Ingenieure, Design & Engineering GmbH\n");
     fprintf(PostStream, "www.drbinde.de\n");
     fprintf(PostStream, "\n");
-    fprintf(PostStream, "         3         1         2        62         2         6\n");//62: Quality
-    fprintf(PostStream, "       -11         0         1         1         1         0         0         0\n");
+    fprintf(PostStream, "         3         1         2        62         2"
+            "         6\n");//62: Quality
+    fprintf(PostStream, "       -11         0         1         1         1"
+            "         0         0         0\n");
     fprintf(PostStream, "       500         0\n");
-    fprintf(PostStream, "  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");
-    fprintf(PostStream, "  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");
+    fprintf(PostStream, "  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00"
+            "  0.00000E+00  0.00000E+00\n");
+    fprintf(PostStream, "  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00"
+            "  0.00000E+00  0.00000E+00\n");
     break ;
   }
 }
@@ -189,6 +211,54 @@ void Gmsh_StringEnd(int Format)
   }
 }
 
+static void printElementNodeData(struct PostSubOperation *PSO_P, int numTimeStep,
+                                 int numComp, int Nb[8], List_T *L[8])
+{
+  int N = 0;
+  for(int i = 0; i < 8; i++) N += Nb[i];
+  if(!N) return;
+  int step = 0;
+  for (int ts = 0; ts < numTimeStep; ts++) {
+    Pos_InitAllSolutions(PSO_P->TimeStep_L, ts);
+    double time = Current.DofData->CurrentSolution->Time;
+    for(int har = 0; har < Current.NbrHar; har++){
+      fprintf(PostStream, "$ElementNodeData\n");
+      fprintf(PostStream, "1\n");
+      fprintf(PostStream, "\"%s\"\n", CurrentName);
+      fprintf(PostStream, "1\n");
+      fprintf(PostStream, "%.16g\n", time);
+      fprintf(PostStream, "4\n");
+      fprintf(PostStream, "%d\n", step);
+      fprintf(PostStream, "%d\n", numComp);
+      fprintf(PostStream, "%d\n", N);
+      fprintf(PostStream, "%d\n", CurrentPartitionNumber);
+      for(int i = 0; i < 8; i++){
+        if(!Nb[i]) continue;
+        int stride = List_Nbr(L[i]) / Nb[i];
+        for(int j = 0; j < List_Nbr(L[i]); j += stride){
+          double *tmp = (double*)List_Pointer(L[i], j);
+          int num = (int)tmp[0];
+          int mult = (stride - 1) / numTimeStep / Current.NbrHar / numComp;
+          if(Flag_BIN){
+            fwrite(&num, sizeof(int), 1, PostStream);
+            fwrite(&mult, sizeof(int), 1, PostStream);
+            fwrite(&tmp[1 + step * mult * numComp], sizeof(double), mult * numComp,
+                   PostStream);          
+          }
+          else{
+            fprintf(PostStream, "%d %d", num, mult);
+            for(int k = 0; k < mult * numComp; k++)
+              fprintf(PostStream, " %.16g", tmp[1 + step * mult * numComp + k]);            
+            fprintf(PostStream, "\n");
+          }
+        }
+      }
+      fprintf(PostStream, "$EndElementNodeData\n");
+      step++;
+    }
+  }
+}
+
 void Format_PostFooter(struct PostSubOperation *PSO_P, int Store)
 {
   List_T  * Iso_L[NBR_MAX_ISO] ;
@@ -197,12 +267,13 @@ void Format_PostFooter(struct PostSubOperation *PSO_P, int Store)
   int       iPost, iNode, iIso, f, iTime, One=1, i, j, NbTimeStep ;
   char      tmp[1024];
   struct PostElement *PE ;
-  
-  if ( (PSO_P->Format == FORMAT_GMSH || PSO_P->Format == FORMAT_GMSH_PARSED) ){
-    
-    if( !(NbTimeStep = List_Nbr(PSO_P->TimeStep_L)) )
-      NbTimeStep = List_Nbr(Current.DofData->Solutions);
-    
+
+  if( !(NbTimeStep = List_Nbr(PSO_P->TimeStep_L)) )
+    NbTimeStep = List_Nbr(Current.DofData->Solutions);
+
+  if ( (PSO_P->Format == FORMAT_GMSH || PSO_P->Format == FORMAT_GMSH_PARSED) && 
+       Flag_GMSH_VERSION != 2 ){
+
     switch(PSO_P->Legend){
       
     case LEGEND_TIME:
@@ -300,12 +371,10 @@ void Format_PostFooter(struct PostSubOperation *PSO_P, int Store)
     for(iIso = 0 ; iIso < NbrIso ; iIso++){
       for(iPost = 0 ; iPost < List_Nbr(Iso_L[iIso]) ; iPost++){
 	PE = *(struct PostElement**)List_Pointer(Iso_L[iIso], iPost) ;
-	Format_PostElement(PSO_P->Format, 0, 0, 
+	Format_PostElement(PSO_P, 0, 0, 
 			   Current.Time, 0, 1, 
 			   Current.NbrHar, PSO_P->HarmonicToTime, 
-			   NULL, PE,
-			   PSO_P->ChangeOfCoordinates,
-			   PSO_P->ChangeOfValues);
+			   NULL, PE);
 	Destroy_PostElement(PE) ;
       }
       List_Delete(Iso_L[iIso]) ;
@@ -326,7 +395,20 @@ void Format_PostFooter(struct PostSubOperation *PSO_P, int Store)
     fprintf(PostStream, "};\n") ;
     break ;
   case FORMAT_GMSH :
-    if(Flag_BIN){ /* bricolage */
+    if(Flag_GMSH_VERSION == 2){
+      int NS[8] = {NbSP, NbSL, NbST, NbSQ, NbSS, NbSH, NbSI, NbSY};
+      List_T *LS[8] = {SP, SL, ST, SQ, SS, SH, SI, SY};
+      printElementNodeData(PSO_P, NbTimeStep, 1, NS, LS);
+
+      int NV[8] = {NbVP, NbVL, NbVT, NbVQ, NbVS, NbVH, NbVI, NbVY};
+      List_T *LV[8] = {VP, VL, VT, VQ, VS, VH, VI, VY};
+      printElementNodeData(PSO_P, NbTimeStep, 3, NV, LV);
+
+      int NT[8] = {NbTP, NbTL, NbTT, NbTQ, NbTS, NbTH, NbTI, NbTY};
+      List_T *LT[8] = {TP, TL, TT, TQ, TS1, TH, TI, TY};
+      printElementNodeData(PSO_P, NbTimeStep, 9, NT, LT);
+    }
+    else if(Flag_BIN){ /* bricolage */
       fprintf(PostStream, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
 	      "%d %d %d %d %d %d %d %d %d %d %d 0 0\n",
 	      List_Nbr(TimeValue_L),
@@ -350,7 +432,7 @@ void Format_PostFooter(struct PostSubOperation *PSO_P, int Store)
       List_WriteToFile(SQ, PostStream, f); List_WriteToFile(VQ, PostStream, f);
       List_WriteToFile(TQ, PostStream, f); 
       List_WriteToFile(SS, PostStream, f); List_WriteToFile(VS, PostStream, f);
-      List_WriteToFile(TS, PostStream, f);
+      List_WriteToFile(TS1, PostStream, f);
       List_WriteToFile(SH, PostStream, f); List_WriteToFile(VH, PostStream, f);
       List_WriteToFile(TH, PostStream, f);
       List_WriteToFile(SI, PostStream, f); List_WriteToFile(VI, PostStream, f);
@@ -393,19 +475,25 @@ void  Format_Unv(int Num_Element, int NbrNodes, struct Value *Value)
   switch (Value[0].Type) {
     
   case SCALAR :
-    //Erste Zeile: Elementnummer, 1:Data present for all nodes, Number of nodes on elements, Number of data values per node
-    fprintf(PostStream, "%10i         1         %i         3\n", Num_Element, NbrNodes);
+    //Erste Zeile: Elementnummer, 1:Data present for all nodes, Number
+    //of nodes on elements, Number of data values per node
+    fprintf(PostStream, "%10i         1         %i         3\n", 
+            Num_Element, NbrNodes);
     for(i = 0 ; i < NbrNodes ; i++){
       //fprintf(PostStream, "%13.5E\n", Value[i].Val[0]);
-      fprintf(PostStream, "%13.5E%13.5E%13.5E\n", Value[i].Val[0], Value[i].Val[1], Value[i].Val[2]);
+      fprintf(PostStream, "%13.5E%13.5E%13.5E\n", Value[i].Val[0],
+              Value[i].Val[1], Value[i].Val[2]);
     }
     break ;
 
   case VECTOR :
-    //Erste Zeile: Elementnummer, 1:Data present for all nodes, Number of nodes on elements, Number of data values per node
-    fprintf(PostStream, "%10i         1         %i         3\n", Num_Element, NbrNodes);
+    //Erste Zeile: Elementnummer, 1:Data present for all nodes, Number
+    //of nodes on elements, Number of data values per node
+    fprintf(PostStream, "%10i         1         %i         3\n", 
+            Num_Element, NbrNodes);
     for(i = 0 ; i < NbrNodes ; i++){
-      fprintf(PostStream, "%13.5E%13.5E%13.5E\n", Value[i].Val[0], Value[i].Val[1], Value[i].Val[2]);
+      fprintf(PostStream, "%13.5E%13.5E%13.5E\n", Value[i].Val[0], 
+              Value[i].Val[1], Value[i].Val[2]);
     }
     break ;
     
@@ -566,8 +654,9 @@ void  Format_GmshParsed(double Time, int TimeStep, int NbTimeStep, int NbHarmoni
 }
 
 void  Format_Gmsh(double Time, int TimeStep, int NbTimeStep, int NbHarmonic, 
-		  int HarmonicToTime, int Type, int NbrNodes, 
-		  double *x, double *y, double *z, struct Value *Value)
+		  int HarmonicToTime, int Type, int ElementNum, int NbrNodes, 
+		  double *x, double *y, double *z, struct Value *Value, 
+                  struct PostSubOperation *PSO_P, int Store)
 {
   int            i,j,k;
   double         TimeMH ;
@@ -595,7 +684,7 @@ void  Format_Gmsh(double Time, int TimeStep, int NbTimeStep, int NbHarmonic,
     if(!TQ) TQ = List_Create(100,1000,sizeof(double)); else List_Reset(TQ);
     if(!SS) SS = List_Create(100,1000,sizeof(double)); else List_Reset(SS);
     if(!VS) VS = List_Create(100,1000,sizeof(double)); else List_Reset(VS);
-    if(!TS) TS = List_Create(100,1000,sizeof(double)); else List_Reset(TS);
+    if(!TS1) TS1 = List_Create(100,1000,sizeof(double)); else List_Reset(TS1);
     if(!SH) SH = List_Create(100,1000,sizeof(double)); else List_Reset(SH);
     if(!VH) VH = List_Create(100,1000,sizeof(double)); else List_Reset(VH);
     if(!TH) TH = List_Create(100,1000,sizeof(double)); else List_Reset(TH);
@@ -630,9 +719,15 @@ void  Format_Gmsh(double Time, int TimeStep, int NbTimeStep, int NbHarmonic,
       case QUADRANGLE_2: Current_L = SQ ; NbSQ++ ; break ;
       case QUADRANGLE_2_8N: Current_L = SQ ; NbSQ++ ; break ;
       }
-      for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &x[i]);
-      for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &y[i]);
-      for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &z[i]);
+      if(Flag_GMSH_VERSION != 2){
+        for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &x[i]);
+        for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &y[i]);
+        for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &z[i]);
+      }
+      else{
+        double tmp = ElementNum;
+        List_Add(Current_L, &tmp);
+      }
     }
     if (HarmonicToTime == 1)
       for(k = 0 ; k < NbHarmonic ; k++){
@@ -667,9 +762,15 @@ void  Format_Gmsh(double Time, int TimeStep, int NbTimeStep, int NbHarmonic,
       case QUADRANGLE_2: Current_L = VQ ; NbVQ++ ; break ;
       case QUADRANGLE_2_8N: Current_L = VQ ; NbVQ++ ; break ;
       }
-      for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &x[i]);
-      for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &y[i]);
-      for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &z[i]);
+      if(Flag_GMSH_VERSION != 2){
+        for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &x[i]);
+        for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &y[i]);
+        for(i = 0 ; i < NbrNodes ; i++) List_Add(Current_L, &z[i]);
+      }
+      else{
+        double tmp = ElementNum;
+        List_Add(Current_L, &tmp);
+      }
     }
     if (HarmonicToTime == 1)
       for(k = 0 ; k < NbHarmonic ; k++){
@@ -696,6 +797,16 @@ void  Format_Gmsh(double Time, int TimeStep, int NbTimeStep, int NbHarmonic,
     break;
     
   }
+
+  // reduce memory requirements by automatically partitioning large
+  // output views into chunks not larger than 1Gb
+  if(Flag_GMSH_VERSION == 2 && TimeStep == NbTimeStep - 1 && 
+     List_Nbr(Current_L) > 1024 * 1024 * 1024 / sizeof(double)){
+    Format_PostFooter(PSO_P, Store);
+    CurrentPartitionNumber++;
+    Gmsh_StartNewView = 1;
+  }
+
 }
 
 /* ------------------------------------------------------------------------ */
@@ -915,12 +1026,10 @@ void  Format_Adapt(double * Dummy)
 /*  F o r m a t _ P o s t E l e m e n t                                     */
 /* ------------------------------------------------------------------------ */
 
-void  Format_PostElement(int Format, int Contour, int Store, 
+void  Format_PostElement(struct PostSubOperation *PSO_P, int Contour, int Store, 
 			 double Time, int TimeStep, int NbTimeStep, 
 			 int NbrHarmonics, int HarmonicToTime, double *Dummy,
-			 struct PostElement * PE, 
-			 int *ChangeOfCoordinates,
-			 List_T *ChangeOfValues)
+			 struct PostElement * PE)
 {
   int    i, j, k, l, Num_Element ;
   struct PostElement  * PE2 ;
@@ -951,30 +1060,30 @@ void  Format_PostElement(int Format, int Contour, int Store,
     return ;
   }
 
-  if(ChangeOfCoordinates && ChangeOfCoordinates[0] >= 0){
+  if(PSO_P->ChangeOfCoordinates && PSO_P->ChangeOfCoordinates[0] >= 0){
     for(i=0 ; i<PE->NbrNodes ; i++){
       Current.x = PE->x[i];
       Current.y = PE->y[i];
       Current.z = PE->z[i];
       for(j = 0; j<9 ; j++) Current.Val[j] = PE->Value[i].Val[j];
-      Get_ValueOfExpressionByIndex(ChangeOfCoordinates[0], NULL, 0., 0., 0., &Value) ; 
+      Get_ValueOfExpressionByIndex(PSO_P->ChangeOfCoordinates[0], NULL, 0., 0., 0., &Value) ; 
       PE->x[i] = Value.Val[0];
-      Get_ValueOfExpressionByIndex(ChangeOfCoordinates[1], NULL, 0., 0., 0., &Value) ; 
+      Get_ValueOfExpressionByIndex(PSO_P->ChangeOfCoordinates[1], NULL, 0., 0., 0., &Value) ; 
       PE->y[i] = Value.Val[0];
-      Get_ValueOfExpressionByIndex(ChangeOfCoordinates[2], NULL, 0., 0., 0., &Value) ; 
+      Get_ValueOfExpressionByIndex(PSO_P->ChangeOfCoordinates[2], NULL, 0., 0., 0., &Value) ; 
       PE->z[i] = Value.Val[0];
     }
   }
 
-  if(ChangeOfValues && List_Nbr(ChangeOfValues) > 0){
+  if(PSO_P->ChangeOfValues && List_Nbr(PSO_P->ChangeOfValues) > 0){
     for(i=0 ; i<PE->NbrNodes ; i++){
       Current.x = PE->x[i];
       Current.y = PE->y[i];
       Current.z = PE->z[i];
       for(k=0 ; k<Current.NbrHar ; k++){
 	for(j = 0; j<9 ; j++) Current.Val[j] = PE->Value[i].Val[MAX_DIM*k+j];
-	for(l=0 ; l<List_Nbr(ChangeOfValues) ; l++){
-	  Get_ValueOfExpressionByIndex(*(int*)List_Pointer(ChangeOfValues, l), 
+	for(l=0 ; l<List_Nbr(PSO_P->ChangeOfValues) ; l++){
+	  Get_ValueOfExpressionByIndex(*(int*)List_Pointer(PSO_P->ChangeOfValues, l), 
 				       NULL, 0., 0., 0., &Value) ; 
 	  PE->Value[i].Val[MAX_DIM*k+l] = Value.Val[0];
 	}
@@ -982,7 +1091,7 @@ void  Format_PostElement(int Format, int Contour, int Store,
     }
   }
 
-  switch(Format){
+  switch(PSO_P->Format){
   case FORMAT_GMSH_PARSED :
     Format_GmshParsed(Time, TimeStep, NbTimeStep, NbrHarmonics, HarmonicToTime,
 		      PE->Type, PE->NbrNodes, PE->x, PE->y, PE->z, 
@@ -992,10 +1101,10 @@ void  Format_PostElement(int Format, int Contour, int Store,
     Format_Unv(Num_Element, PE->NbrNodes, PE->Value) ;
     break ;
   case FORMAT_GMSH :
-    if(Flag_BIN){/* bricolage */
+    if(Flag_GMSH_VERSION == 2 || Flag_BIN){ /* bricolage */
       Format_Gmsh(Time, TimeStep, NbTimeStep, NbrHarmonics, HarmonicToTime,
-		  PE->Type, PE->NbrNodes, PE->x, PE->y, PE->z, 
-		  PE->Value) ;
+		  PE->Type, Num_Element, PE->NbrNodes, PE->x, PE->y, PE->z, 
+		  PE->Value, PSO_P, Store) ;
     }
     else{
       Format_GmshParsed(Time, TimeStep, NbTimeStep, NbrHarmonics, HarmonicToTime,
@@ -1004,14 +1113,14 @@ void  Format_PostElement(int Format, int Contour, int Store,
     }
     break ;
   case FORMAT_GNUPLOT :
-    Format_Gnuplot(Format, Time, TimeStep, NbTimeStep, NbrHarmonics, HarmonicToTime,
+    Format_Gnuplot(PSO_P->Format, Time, TimeStep, NbTimeStep, NbrHarmonics, HarmonicToTime,
 		   PE->Type, Num_Element, PE->NbrNodes, PE->x, PE->y, PE->z, Dummy, 
 		   PE->Value) ;
     break ;
   case FORMAT_SPACE_TABLE :
   case FORMAT_TIME_TABLE :
   case FORMAT_SIMPLE_SPACE_TABLE :
-    Format_Tabular(Format, Time, TimeStep, NbTimeStep, NbrHarmonics, HarmonicToTime,
+    Format_Tabular(PSO_P->Format, Time, TimeStep, NbTimeStep, NbrHarmonics, HarmonicToTime,
 		   PE->Type, Num_Element, PE->NbrNodes, PE->x, PE->y, PE->z, Dummy, 
 		   PE->Value) ;
     break ;
@@ -1068,7 +1177,7 @@ void Format_PostValue(int Format, int Flag_Comma, int Group_FunctionType,
       fprintf(PostStream, "%s\n", sstream.str().c_str()) ;
   }
 
-  else if (Format == FORMAT_GMSH) {
+  else if (Format == FORMAT_GMSH && Flag_GMSH_VERSION != 2) {
     if (Group_FunctionType == NODESOF)
       Geo_GetNodesCoordinates(1, &numRegion, &x, &y, &z) ;
     else {
