@@ -325,12 +325,10 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
 	  for(iNode = 0 ; iNode < PE->NbrNodes ; iNode++)
 	    Cal_CopyValue(&CumulativeValues[iTime], &PE->Value[iNode]);
 	  if(!Store)
-	    Format_PostElement(PostSubOperation_P->Format, PostSubOperation_P->Iso, 0,
+	    Format_PostElement(PostSubOperation_P, PostSubOperation_P->Iso, 0,
 			       Current.Time, iTime, NbrTimeStep, 
 			       Current.NbrHar, PostSubOperation_P->HarmonicToTime,
-			       NULL, PE, 
-			       PostSubOperation_P->ChangeOfCoordinates,
-			       PostSubOperation_P->ChangeOfValues);
+			       NULL, PE);
 	}
       }
       else{ /* There is one non-cumulative */
@@ -355,12 +353,10 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
 				     &PE->Value[iNode], &CumulativeValues[iNode]) ;
 	    }
 	    if(!Store)
-	      Format_PostElement(PostSubOperation_P->Format, PostSubOperation_P->Iso, 0,
+	      Format_PostElement(PostSubOperation_P, PostSubOperation_P->Iso, 0,
 				 Current.Time, iTime, NbrTimeStep, 
 				 Current.NbrHar, PostSubOperation_P->HarmonicToTime,
-				 NULL, PE, 
-				 PostSubOperation_P->ChangeOfCoordinates,
-				 PostSubOperation_P->ChangeOfValues);
+				 NULL, PE);
 	  }
 	}
 	else{ /* PRINT_ONREGION: We work on the real mesh */
@@ -387,12 +383,10 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
 	    }      
 
 	    if(!Store)
-	      Format_PostElement(PostSubOperation_P->Format, PostSubOperation_P->Iso, 0,
+	      Format_PostElement(PostSubOperation_P, PostSubOperation_P->Iso, 0,
 				 Current.Time, iTime, NbrTimeStep,
 				 Current.NbrHar, PostSubOperation_P->HarmonicToTime,
-				 NULL, PE, 
-				 PostSubOperation_P->ChangeOfCoordinates,
-				 PostSubOperation_P->ChangeOfValues);
+				 NULL, PE);
 	  }
 	}
       }
@@ -550,12 +544,10 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
 	Dummy[3] = -1. ;
       }
 
-      Format_PostElement(PostSubOperation_P->Format, PostSubOperation_P->Iso, 1,
+      Format_PostElement(PostSubOperation_P, PostSubOperation_P->Iso, 1,
 			 Current.Time, 0, 1, 
 			 Current.NbrHar, PostSubOperation_P->HarmonicToTime,
-			 Dummy, PE, 
-			 PostSubOperation_P->ChangeOfCoordinates,
-			 PostSubOperation_P->ChangeOfValues);
+			 Dummy, PE);
     }
   }
 
@@ -649,12 +641,10 @@ void normvec(double *a)
 	else									\
 	  Cal_CopyValue(&CumulativeValues[iTime],&PE->Value[iNode]);		\
       }										\
-      Format_PostElement(PostSubOperation_P->Format, PostSubOperation_P->Iso,0, \
+      Format_PostElement(PostSubOperation_P, PostSubOperation_P->Iso,0,         \
 			 Current.Time, iTime, NbTimeStep,			\
 			 Current.NbrHar, PostSubOperation_P->HarmonicToTime,	\
-			 NULL, PE,                                              \
-			 PostSubOperation_P->ChangeOfCoordinates,               \
-			 PostSubOperation_P->ChangeOfValues);	                \
+			 NULL, PE);                     	                \
     }										\
   }										\
   for(iPost = 0 ; iPost < List_Nbr(PE_L) ; iPost++)				\
@@ -874,12 +864,10 @@ void  Pos_PrintOnSection(struct PostQuantity     *NCPQ_P,
  if(!NCPQ_P){									\
    for (ts = 0 ; ts < NbTimeStep ; ts++){					\
      PE->Value[0] = CumulativeValues[ts] ;					\
-     Format_PostElement(PSO_P->Format, PSO_P->Iso, 0,				\
+     Format_PostElement(PSO_P, PSO_P->Iso, 0,			        	\
 		        Current.Time, ts, NbTimeStep,				\
                         Current.NbrHar, PSO_P->HarmonicToTime,                  \
-			Normal, PE,                                             \
-			PSO_P->ChangeOfCoordinates,                             \
-			PSO_P->ChangeOfValues);                                 \
+			Normal, PE);                                            \
    }										\
  }										\
  else{										\
@@ -897,12 +885,10 @@ void  Pos_PrintOnSection(struct PostQuantity     *NCPQ_P,
      if(CPQ_P)									\
        Combine_PostQuantity(PSO_P->CombinationType, Order,			\
                             &PE->Value[0], &CumulativeValues[ts]) ;		\
-     Format_PostElement(PSO_P->Format, PSO_P->Iso, 0,				\
+     Format_PostElement(PSO_P, PSO_P->Iso, 0,		        		\
                         Current.Time, ts, NbTimeStep,				\
                         Current.NbrHar, PSO_P->HarmonicToTime,                  \
-			Normal, PE,                                      	\
-			PSO_P->ChangeOfCoordinates,                             \
-			PSO_P->ChangeOfValues);                                 \
+			Normal, PE);                                            \
    }										\
  }
 
@@ -1068,12 +1054,10 @@ void  Pos_PrintOnGrid(struct PostQuantity     *NCPQ_P,
 	      PE2->Value[1].Val[MAX_DIM*k] = ARRAY(i1+1,i2,k,ts) ;
 	      PE2->Value[2].Val[MAX_DIM*k] = ARRAY(i1,i2+1,k,ts) ;
 	    }
-	    Format_PostElement(PSO_P->Format, PSO_P->Iso, 0,
+	    Format_PostElement(PSO_P, PSO_P->Iso, 0,
 			       Current.Time, ts, NbTimeStep,
 			       Current.NbrHar, PSO_P->HarmonicToTime, 
-			       Normal, PE2,
-			       PSO_P->ChangeOfCoordinates,
-			       PSO_P->ChangeOfValues);
+			       Normal, PE2);
 	  }
 	  PE2->x[0] = X[0] + (X[1] - X[0]) * S[2] + (X[2] - X[0]) * S[3] ;
 	  PE2->y[0] = Y[0] + (Y[1] - Y[0]) * S[2] + (Y[2] - Y[0]) * S[3] ;
@@ -1087,12 +1071,10 @@ void  Pos_PrintOnGrid(struct PostQuantity     *NCPQ_P,
 	      PE2->Value[1].Val[MAX_DIM*k] = ARRAY(i1,i2+1,k,ts) ;
 	      PE2->Value[2].Val[MAX_DIM*k] = ARRAY(i1+1,i2,k,ts) ;
 	    }
-	    Format_PostElement(PSO_P->Format, PSO_P->Iso, 0,
+	    Format_PostElement(PSO_P, PSO_P->Iso, 0,
 			       Current.Time, ts, NbTimeStep,
 			       Current.NbrHar, PSO_P->HarmonicToTime, 
-			       Normal, PE2, 
-			       PSO_P->ChangeOfCoordinates,
-			       PSO_P->ChangeOfValues);
+			       Normal, PE2);
 	  }
 	}
       }
@@ -1451,7 +1433,7 @@ void  Pos_PrintGroup(struct PostSubOperation *PostSubOperation_P)
 {
   struct Group        *Group_P;
   struct Geo_Element  *GeoElement;
-  struct PostElement  *SL;
+  struct PostElement  *SL, *ST, *SQ;
   List_T              *Region_L;
   int                  i, NbrGeo, iGeo, *NumNodes;
   double               x [NBR_MAX_NODES_IN_ELEMENT] ;
@@ -1474,6 +1456,8 @@ void  Pos_PrintGroup(struct PostSubOperation *PostSubOperation_P)
 		 PostSubOperation_P->Case.Group.ExtendedGroupIndex);
 
   SL = Create_PostElement(0, LINE, 2, 1) ;
+  ST = Create_PostElement(0, TRIANGLE, 3, 1) ;
+  SQ = Create_PostElement(0, QUADRANGLE, 4, 1) ;
 
   if(!Group_P->ExtendedList) Generate_ExtendedGroup(Group_P) ;
 
@@ -1499,15 +1483,51 @@ void  Pos_PrintGroup(struct PostSubOperation *PostSubOperation_P)
 	    SL->z[0] = z[abs(NumNodes[0])-1]; SL->z[1] = z[abs(NumNodes[1])-1];
 	    SL->Value[0].Type = SL->Value[1].Type = SCALAR ;
 	    SL->Value[0].Val[0] = SL->Value[1].Val[0] = GeoElement->NumEdges[i];
-	    Format_PostElement(PostSubOperation_P->Format, PostSubOperation_P->Iso, 0,
+	    Format_PostElement(PostSubOperation_P, PostSubOperation_P->Iso, 0,
 			       0, 0, 1, 1, 1, 
-			       NULL, SL, 
-			       PostSubOperation_P->ChangeOfCoordinates,
-			       PostSubOperation_P->ChangeOfValues);
+			       NULL, SL);
 	  }
 	}
 	break ;
 
+      case FACETSOFTREEIN :
+	if(!GeoElement->NbrFacets) Geo_CreateFacetsOfElement(GeoElement) ;
+	for(i=0 ; i<GeoElement->NbrFacets ; i++){
+	  if(List_Search(Group_P->ExtendedList, &GeoElement->NumFacets[i], fcmp_absint)){
+	    NumNodes = Geo_GetNodesOfFacetInElement(GeoElement, i) ;
+            if(!NumNodes[3]){ // we have triangle
+              ST->Index = iGeo;
+              ST->x[0] = x[abs(NumNodes[0])-1]; ST->x[1] = x[abs(NumNodes[1])-1];
+              ST->y[0] = y[abs(NumNodes[0])-1]; ST->y[1] = y[abs(NumNodes[1])-1];
+              ST->z[0] = z[abs(NumNodes[0])-1]; ST->z[1] = z[abs(NumNodes[1])-1];
+              ST->x[2] = x[abs(NumNodes[2])-1];
+              ST->y[2] = y[abs(NumNodes[2])-1];
+              ST->z[2] = z[abs(NumNodes[2])-1];
+              ST->Value[0].Type = ST->Value[1].Type = ST->Value[2].Type = SCALAR ;
+              ST->Value[0].Val[0] = ST->Value[1].Val[0] = ST->Value[2].Val[0] = GeoElement->NumFacets[i];
+              Format_PostElement(PostSubOperation_P, PostSubOperation_P->Iso, 0,
+                                 0, 0, 1, 1, 1, 
+                                 NULL, ST);
+            }
+            else{ // we have a quad
+              SQ->Index = iGeo;
+              SQ->x[0] = x[abs(NumNodes[0])-1]; SQ->x[1] = x[abs(NumNodes[1])-1];
+              SQ->y[0] = y[abs(NumNodes[0])-1]; SQ->y[1] = y[abs(NumNodes[1])-1];
+              SQ->z[0] = z[abs(NumNodes[0])-1]; SQ->z[1] = z[abs(NumNodes[1])-1];
+              SQ->x[2] = x[abs(NumNodes[2])-1]; SQ->x[3] = x[abs(NumNodes[3])-1];
+              SQ->y[2] = y[abs(NumNodes[2])-1]; SQ->y[3] = y[abs(NumNodes[3])-1];
+              SQ->z[2] = z[abs(NumNodes[2])-1]; SQ->z[3] = z[abs(NumNodes[3])-1];
+              SQ->Value[0].Type = SQ->Value[1].Type = SQ->Value[2].Type = SQ->Value[3].Type = SCALAR ;
+              SQ->Value[0].Val[0] = SQ->Value[1].Val[0] = SQ->Value[2].Val[0] = 
+                SQ->Value[3].Val[0] = GeoElement->NumFacets[i];
+              Format_PostElement(PostSubOperation_P, PostSubOperation_P->Iso, 0,
+                                 0, 0, 1, 1, 1, 
+                                 NULL, SQ);
+            }
+          }
+	}
+	break ;
+
       default :
 	Msg::Error("Print function not implemented for this kind of Group");
 	break ;
